Attribute VB_Name = "modResizeForm"
Option Compare Database
Option Explicit

Dim strProc As String
Public blnResize As Boolean

Public sngZoom As Single, sngOldZoom As Single
Public ctrl As Access.Control, frm As Access.Form

Public lngOrigWidth As Long, lngOrigHeight As Long, lngNewWidth As Long, lngNewHeight As Long, strDir As String
Public lngOldWidth As Long, lngOldHeight As Long 'v3.90

Public lngHeaderHeight As Long, lngFooterHeight As Long, lngDetailHeight As Long 'v3.91

' Module    : modResizeForm
' DateTime  : 27/01/2003
' Author    :   Jamie Czernik BSc {jamiessoftware@blueyonder.co.uk}
' Purpose   :  Resizes Microsoft Access forms to fit the current screen resolution.
' Please    :   If you find this code useful please consider placing a link to my web site
'                   (http://www.jamiessoftware.tk) on your own web site so that others may  benefit as well.

' Credits   :   The original modResizeForm module was created by Jamie Czernik
'                    Contains improvement suggestions/updates from:  Dr. Martin Dumskyj & Nathan Carroll.
'                    Minor changes by Jeff Blumsom to set a maximum factor of 1.28
'                    Contains enhancements which fixed some problems in combo boxes, list boxes, and tab controls from: Myke Myers.

'                    Significantly modified by Colin Riddington between 2006 & 2022:
'                    Support for widescreen monitors and tabbed documents display
'                    Added code to zoom forms & fix issues with over-sized forms
'                    Added support for datasheet forms, split forms
'                    Added code to fix issues where resizing exceeds max form width (integer twips limit) on hi-res monitors
'                    Added support for high resolution 4K monitors e.g. 3840x2160, split screen e.g. 2 x 1680x1050 and portrait monitors e.g. 1080x1920
'                    New code to detect size & resolution of secondary monitors (as well as the primary monitor)
'                    Added code to refresh display of open form when resolution is changed e.g. moved to new monitor
'                    NOTE: this required a major rewrite of several procedures including: Resize/ResizeForm/ZoomForm
'                    New code to allow 'resiziing' of images on command buttons
'                    New code to allow stretch/shrink of forms on the fly using a mouse

'                    Modified 18/02/2023 to run in A2007 by removing support for navigation forms & navigation pane code
'
' Last Updated  18 Feb 2023

'---------------------------------------------------------------------------------------
'-----------------------------MODULE CONSTANTS & VARIABLES------------------------------
Private Const DESIGN_HORZRES As Long = 800 '1280  '1680 '1366 '1024 '800 '600
'<- CHANGE THE VALUE ABOVE TO THE RESOLUTION YOU DESIGNED YOUR FORM
'(e.g. 800 X 600 -> 800)
Private Const DESIGN_VERTRES As Long = 600 '1080 '1050 '900 '768 '600 '480
'<- YOU CAN CHANGE THIS VALUE TO THE RESOLUTION YOU DESIGNED YOUR FORMS IN.
'HOWEVER THE VALUE IS NO LONGER USED SO IT CAN BE IGNORED
'(e.g. 800 X 600 -> 600)
Private Const DESIGN_PIXELS As Long = 96
'<- CHANGE THE VALUE ABOVE TO THE DPI SETTING YOU DESIGNED YOUR FORM IN.
'(If in doubt do not alter the mlngDesignPixels setting.)

Private Const WM_HORZRES As Long = 8
Private Const WM_VERTRES As Long = 10
Private Const WM_LOGPIXELSX As Long = 88
Private Const TITLEBAR_PIXELS As Long = 18
Private Const COMMANDBAR_PIXELS As Long = 26
Private Const COMMANDBAR_LEFT As Long = 0
Private Const COMMANDBAR_TOP As Long = 1
Private OrigWindow As tWindow 'Module level variable which holds the original window dimensions before resize.
Private ResizedWindow As tWindow 'Module level variable which holds the new window dimensions after resize.

Private Type tRect
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Private Type tDisplay
    Height As Long
    Width As Long
    DPI As Long
End Type

Private Type tWindow
    Height As Long
    Width As Long
End Type

Dim NewRes As tDisplay
Dim FormFactor As String
Dim sngFormFactor As Single 'CR v4590W
Dim strResolution As String 'CR v4591W

'-------------------------- END MODULE CONSTANTS & VARIABLES----------------------------

'------------------------------------API DECLARATIONS-----------------------------------
    Private Declare Function GetDeviceCaps Lib "gdi32" (ByVal hDC As Long, ByVal nIndex As Long) As Long
    
    Private Declare Function GetDesktopWindow Lib "user32" () As Long
    
    Private Declare Function GetDC Lib "user32" (ByVal hWnd As Long) As Long
    
    Private Declare Function ReleaseDC Lib "user32" (ByVal hWnd As Long, ByVal hDC As Long) As Long
    
    Private Declare Function GetWindowRect Lib "user32.dll" (ByVal hWnd As Long, lpRect As tRect) As Long
    
    Private Declare Function MoveWindow Lib "user32.dll" (ByVal hWnd As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, _
        ByVal nHeight As Long, ByVal bRepaint As Long) As Long
    
    Private Declare Function IsZoomed Lib "user32.dll" (ByVal hWnd As Long) As Long
    
    Private Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long) As Long

    Private Const GWL_EXSTYLE As Long = -20
    Private Const WS_EX_DLGMODALFRAME As Long = &H1
'--------------------------------- END API DECLARATIONS----------------------------------


'---------------------------------------------------------------------------------------
' Procedure : getScreenResolution
' DateTime  : 27/01/2003
' Author    : Jamie Czernik, modified by Jeff Blumsom/Colin riddington
' Purpose   : Function returns the current height, width and dpi.
'---------------------------------------------------------------------------------------
Public Sub GetScreenResolution()

Dim hDCcaps As Long
Dim lngRtn As Long

On Error Resume Next

    'API call get current resolution
    hDCcaps = GetDC(0) 'get display context for desktop (hwnd = 0)
    With NewRes
        .Height = GetDeviceCaps(hDCcaps, WM_VERTRES)
        .Width = GetDeviceCaps(hDCcaps, WM_HORZRES)
        .DPI = GetDeviceCaps(hDCcaps, WM_LOGPIXELSX)
    '============================================================
    'modifications by Jeff Blumsom/Colin Riddington
        Select Case (.Width / .Height)
            Case 4 / 3
                FormFactor = "4:3"
            Case 5 / 4
                FormFactor = "5:4"
            Case Is > 4 / 3
                FormFactor = "Widescreen"
            Case Is < 1 'v3.80
                FormFactor = "Portrait"
            Case Else 'v3.80
                FormFactor = "Other" 'e.g. split screen
        End Select
        
      '  Debug.Print .Width & " " & .Height & " " & FormFactor & " " & .DPI
               
        sngFormFactor = .Width / .Height
       ' Debug.Print sngFormFactor
        
        strResolution = .Width & "*" & .Height
        
    '============================================================
    End With
    
    lngRtn = ReleaseDC(0, hDCcaps) 'release display context
    
    
End Sub

Public Function GetFormFactor()

On Error GoTo Err_Handler

    GetFormFactor = Round(sngFormFactor, 2)
    'Debug.Print GetFormFactor

Exit_Handler:
    Exit Function

Err_Handler:
    strProc = "GetFormFactor"
    MsgBox "Error " & Err.Number & " in " & strProc & " procedure : " & Err.Description
    Resume Exit_Handler

End Function

Public Function GetResolution()

On Error GoTo Err_Handler

    GetResolution = strResolution

Exit_Handler:
    Exit Function

Err_Handler:
    strProc = "GetResolution"
    MsgBox "Error " & Err.Number & " in " & strProc & " procedure : " & Err.Description
    Resume Exit_Handler

End Function
Public Function GetHorizontalResolution()

'no longer used

On Error GoTo Err_Handler

    'discard vertical resolution & formfactor
    GetHorizontalResolution = Left(strResolution, InStr(strResolution, "*") - 1)
  '  Debug.Print GetHorizontalResolution

Exit_Handler:
    Exit Function

Err_Handler:
    strProc = "GetHorizontalResolution"
    MsgBox "Error " & Err.Number & " in " & strProc & " procedure : " & Err.Description
    Resume Exit_Handler
    
End Function
Public Function GetVerticalResolution()

'no longer used

On Error GoTo Err_Handler

    'discard horizontal resolution
    GetVerticalResolution = Mid(strResolution, InStr(strResolution, "*") + 1)
   ' Debug.Print GetVerticalResolution
    'discard formfactor
    If InStr(GetVerticalResolution, " ") > 0 Then
        GetVerticalResolution = Left(GetVerticalResolution, InStr(GetVerticalResolution, " ") - 1)
    End If
   ' Debug.Print GetVerticalResolution

Exit_Handler:
    Exit Function

Err_Handler:
    strProc = "GetVerticalResolution"
    MsgBox "Error " & Err.Number & " in " & strProc & " procedure : " & Err.Description
    Resume Exit_Handler
    
End Function
Public Function GetScreenShape()

On Error GoTo Err_Handler

GetScreenShape = FormFactor

Exit_Handler:
    Exit Function

Err_Handler:
    strProc = "GetScreenShape"
    MsgBox "Error " & Err.Number & " in " & strProc & " procedure : " & Err.Description
    Resume Exit_Handler
    
End Function

'---------------------------------------------------------------------------------------
' Procedure :  getFactor
' DateTime  :  27/01/2003
' Author    :  Jamie Czernik, modified by Jeff Blumsom/Colin Riddington
' Purpose   :  Function returns the value that the form's/control's height, width, top &
'                    left should be multiplied by to fit the current screen resolution.
'---------------------------------------------------------------------------------------
Public Function GetFactor() As Single

Dim sngFactorP As Single

On Error Resume Next

    If NewRes.DPI <> 0 Then
        sngFactorP = DESIGN_PIXELS / NewRes.DPI
    Else
        sngFactorP = 1 'error with dpi reported so assume 96 dpi
    End If
    '============================================================
    'modifications by Jeff Blumsom / Colin Riddington
    Select Case FormFactor
        Case "4:3"
            GetFactor = (NewRes.Width / DESIGN_HORZRES) * sngFactorP
        Case "5:4"
            GetFactor = (NewRes.Width / DESIGN_HORZRES) * sngFactorP
        Case "Widescreen"
            GetFactor = ((4 / 3) * NewRes.Height / DESIGN_HORZRES) * sngFactorP
            'Modified by Colin Riddington v3.6 23/11/2021
            'This fixes very rare issue where value of GetFactor < 1 causes forms to shrink not grow!
            'NOT WIDELY TESTED AS YET!
            If GetFactor < 1 Then GetFactor = (NewRes.Width / DESIGN_HORZRES) * sngFactorP
            
         '======ADDED v3.80=============
        Case "Portrait"    'e.g. 1080x1920
            GetFactor = (NewRes.Width / DESIGN_HORZRES) * sngFactorP
        
        Case "Other" 'e.g. split screen @1720x1440
            GetFactor = (NewRes.Width / DESIGN_HORZRES) * sngFactorP
    End Select
    '============================================================
       
   '  Debug.Print sngFactorP, GetFactor, FormFactor
     
End Function

'---------------------------------------------------------------------------------------
' Procedure : ReSizeForm
' DateTime  : 27/01/2003
' Author    : Jamie Czernik / Colin Riddington
' Purpose   : Routine should be called on a form's onOpen or onLoad event.

' Modifications: Jeff Blumson / Colin Riddington
' Significantly modified by Colin Riddington 2006-2022
'---------------------------------------------------------------------------------------

Public Sub ReSizeForm(frm As Access.Form)

Dim rectWindow As tRect, sngFactor As Single
Dim lngWidth As Long, lngHeight As Long

On Error Resume Next
    GetScreenResolution
    sngFactor = GetFactor 'local function
    If NewRes.Width <> DESIGN_HORZRES Then 'no resize necessary
        Resize sngFactor, frm 'local procedure
        
'the following code controls the positioning of pop-up forms
'but only if the form tag is null.  This allows more control where it causes a problem
        If IsZoomed(frm.hWnd) = 0 Then            'Don't change window settings for maximised form.
            Access.DoCmd.RunCommand acCmdAppMaximize    'Max Access Window
            Call GetWindowRect(frm.hWnd, rectWindow)
            With rectWindow
                lngWidth = .Right - .Left
                lngHeight = .Bottom - .Top
            End With
            If Nz(frm.Tag, 1) <> 1 Then
                Call MoveWindow(frm.hWnd, ((NewRes.Width - _
                (sngFactor * lngWidth)) / 2) - getLeftOffset, _
                ((NewRes.Height - (sngFactor * lngHeight)) / 2) - getTopOffset, _
                lngWidth * sngFactor, lngHeight * sngFactor, 1)
            End If
        End If
    End If
    
    'Modification by Colin Riddington 13/03/2019
   'UseMDIMode property =1 (overlapping windows) or = 0 (tabbed documents)
   'next section fixes display issue for users of tabbed documents (MDIMode=0)
    If CurrentDb.Properties("UseMDIMode") = 0 Then
        MaximizeNavigationPane
        DoEvents
        MinimizeNavigationPane
    End If
        
End Sub

Sub FixFormSize()

'---------------------------------------------------------------------------------------
' Procedure :            FixFormSize
' DateTime  :           23/09/2015
' Author       :           Colin Riddington
' Purpose     :           Can be used to restore size of forms which have become 'over-sized'
'                               Or just use frmFormUnresizer (no need to open in design view first)
'---------------------------------------------------------------------------------------

On Error GoTo Err_Handler
'Make sure the form you need to un-resize is open in design view before running this process
    Dim frm As Access.Form
    Set frm = Forms!frmMain 'modify form name as necessary
    
    UnReSizeForm frm
    'ReSizeForm frm

Exit_Handler:
    Exit Sub

Err_Handler:
    strProc = "FixFormSize"
    MsgBox "Error " & Err.Number & " in " & strProc & " procedure : " & Err.Description
    Resume Exit_Handler
    
End Sub

Public Sub UnReSizeForm(frm As Access.Form)

'---------------------------------------------------------------------------------------
' Procedure :            UnReSizeForm
' DateTime  :           27/01/2003 with many updates
' Author       :           Jamie Czernik / Colin Riddington
' Purpose     :           Used to unresize forms where needed e.g. before changing zoom

' Modifications:       Jeff Blumson / Colin Riddington

'Significantly modified by Colin Riddington 2006-2022
'---------------------------------------------------------------------------------------

Dim rectWindow As tRect
Dim lngWidth As Long
Dim lngHeight As Long
Dim sngFactor As Single

On Error Resume Next

    GetScreenResolution
    sngFactor = 1 / GetFactor 'local function
    If NewRes.Width <> DESIGN_HORZRES Then 'no resize necessary
        Resize sngFactor, frm 'local procedure
        If IsZoomed(frm.hWnd) = 0 Then 'Don't change window settings for max'd form.
            DoCmd.RunCommand acCmdAppMaximize 'Max Access Window
            Call GetWindowRect(frm.hWnd, rectWindow)
            With rectWindow
                lngWidth = .Right - .Left
                lngHeight = .Bottom - .Top
            End With
            Call MoveWindow(frm.hWnd, ((NewRes.Width - _
            (sngFactor * lngWidth)) / 2) - getLeftOffset, _
            ((NewRes.Height - (sngFactor * lngHeight)) / 2) - getTopOffset, _
            lngWidth * sngFactor, lngHeight * sngFactor, 1)
        End If
    End If
    
End Sub

Public Sub Resize(sngFactor As Single, frm As Access.Form)

'---------------------------------------------------------------------------------------
' Procedure :            Resize
' DateTime  :           27/01/2003 with many updates by Colin Riddington
' Author       :           Jamie Czernik
' Purpose     :           Routine re-scales the form sections and controls.

'Modifications by Colin Riddington:
'04/03/2019  :         Fixed issue with font size issue on some controls
'02/10/2021 :         Updated code to handle acNavigationControl & acNavigationButton - DELETED HERE
'09/11/2022 :         Added check to ensure resized form width doesn't exceed integer limit - needed for high resolution monitors
'18/02/2023:          Adapted for A2007 - removed code related to acNavigationButton, acNavigationControl (both added in A2010)
'---------------------------------------------------------------------------------------

Dim ctl As Access.Control, sngNCW As Single

On Error Resume Next
    
    With frm
         'First check resized form doesn't exceed maximum form width (32767 twips) to prevent display issues
        'Set limit to 32000 to allow some leeway -  then if necessary adjust scaling factor so it fits
        If .Width * sngFactor > 32000 Then sngFactor = 32000 / .Width
    
        'Now resize width/height for each section:
        .Width = .Width * sngFactor
       .Section(Access.acHeader).Height = .Section(Access.acHeader).Height * sngFactor
       .Section(Access.acDetail).Height = .Section(Access.acDetail).Height * sngFactor
       .Section(Access.acFooter).Height = .Section(Access.acFooter).Height * sngFactor
    End With
        
    'Resize and locate each control:
    For Each ctl In frm.Controls
        If ctl.ControlType <> acPage Then 'Ignore pages in TAB controls
            With ctl
                .Left = .Left * sngFactor
                .Top = .Top * sngFactor
                .Height = .Height * sngFactor
                .Width = .Width * sngFactor
               
                
                'CR 04/03/2019 - next line moved back to individual control type section
                ' .FontSize = .FontSize * sngFactor
                
                'Enhancement by Myke Myers (NOT Austin Powers!) & Colin Riddington------------------>
                'Fix certain combo box, list box and tab control properties:
                Select Case .ControlType
                    Case acLabel, acCommandButton, acTextBox, acToggleButton
                         'increase font size proportionately
                         .FontSize = .FontSize * sngFactor
                    Case acListBox
                        'as above & increase width of each column proportionately
                         .FontSize = .FontSize * sngFactor
                        .ColumnWidths = adjustColumnWidths(.ColumnWidths, sngFactor)
                    Case acComboBox
                        'as above & increase list width proportionately
                        .FontSize = .FontSize * sngFactor
                        .ColumnWidths = adjustColumnWidths(.ColumnWidths, sngFactor)
                        .ListWidth = .ListWidth * sngFactor
                    Case acTabCtl
                        'as for textbox & increase width/height of each tab proportionately
                        .FontSize = .FontSize * sngFactor
                        .TabFixedWidth = .TabFixedWidth * sngFactor
                        .TabFixedHeight = .TabFixedHeight * sngFactor
                    Case Else
                        'no code needed for other control types
                        'acRectangle, acCheckBox, acImage, acLine, acPageBreak, acSubform
                        'acOptionButton, acOptionGroup, acObjectFrame, acBoundObjectFrame
                        
                End Select
                '------------------------------------> End enhancement by Myke Myers / Colin Riddington
            End With
        End If
    Next ctl
    
End Sub

'---------------------------------------------------------------------------------------
' Procedure : getTopOffset
' DateTime  : 27/01/2003
' Author    : Jamie Czernik
' Purpose   : Function returns the total size in pixels of menu/toolbars at the top of
'             the Access window allowing the form to be positioned in the centre of the
'             screen.
'---------------------------------------------------------------------------------------
Private Function getTopOffset() As Long

Dim cmdBar As Object
Dim lngI As Long

On Error GoTo Err_Handler

     For Each cmdBar In Application.CommandBars
        If ((cmdBar.Visible = True) And (cmdBar.Position = COMMANDBAR_TOP)) Then
            lngI = lngI + 1
        End If
     Next cmdBar
     getTopOffset = (TITLEBAR_PIXELS + (lngI * COMMANDBAR_PIXELS))

Exit_Handler:
    Exit Function
    
Err_Handler:
    'Assume only 1 visible command bar plus the title bar:
    getTopOffset = TITLEBAR_PIXELS + COMMANDBAR_PIXELS
    
    strProc = "getTopOffset"
    MsgBox "Error " & Err.Number & " in " & strProc & " procedure : " & Err.Description
    Resume Exit_Handler
     
End Function

'---------------------------------------------------------------------------------------
' Procedure : getLeftOffset
' DateTime  : 27/01/2003
' Author    : Jamie Czernik
' Purpose   : Function returns the total size in pixels of menu/toolbars at the left of
'             the Access window allowing the form to be positioned in the centre of the
'             screen.
'---------------------------------------------------------------------------------------
Private Function getLeftOffset() As Long

Dim cmdBar As Object
Dim lngI As Long

On Error GoTo Err_Handler

     For Each cmdBar In Application.CommandBars
        If ((cmdBar.Visible = True) And (cmdBar.Position = COMMANDBAR_LEFT)) Then
            lngI = lngI + 1
        End If
     Next cmdBar
     getLeftOffset = (lngI * COMMANDBAR_PIXELS)

Exit_Handler:
    Exit Function
    
Err_Handler:
    'Assume no visible command bars:
    getLeftOffset = 0
    
    strProc = "getLeftOffset"
    MsgBox "Error " & Err.Number & " in " & strProc & " procedure : " & Err.Description
    Resume Exit_Handler
     
End Function
 
'---------------------------------------------------------------------------------------
' Procedure : adjustColumnWidths
' DateTime  : 27/01/2003
' Author    : Myke Myers [Split() replacement for Access 97 by Jamie Czernik]
' Purpose   : Adjusts column widths for list boxes and combo boxes.
'---------------------------------------------------------------------------------------
Private Function adjustColumnWidths(strColumnWidths As String, sngFactor As Single) _
As String

Dim astrColumnWidths() As String
Dim strTemp As String
Dim lngI As Long
Dim lngJ As Long

On Error GoTo Err_Handler

    'Get the column widths
    ReDim astrColumnWidths(0)

   astrColumnWidths = Split(strColumnWidths, ";")
    
    Do Until lngI > UBound(astrColumnWidths) 'Loop through all divisions
        strTemp = strTemp & CSng(astrColumnWidths(lngI)) * sngFactor & ";"
        lngI = lngI + 1
    Loop
    adjustColumnWidths = strTemp
    
Exit_Handler:
    Exit Function

Err_Handler:
    If Err = 13 Then Resume Next
    strProc = "adjustColumnWidths"
    MsgBox "Error " & Err.Number & " in " & strProc & " procedure : " & Err.Description
    Resume Exit_Handler
    
End Function

'---------------------------------------------------------------------------------------
' Procedure : GetOrigWindow
' DateTime  : 27/01/2003
' Author    : Jamie Czernik
' Purpose   : Routine stores the original window dimensions before resizing
'             Call it when form loads. (before calling ResizeForm Me!).
'             Call it in Form_Load()
'            [More info in "Important Points" - point 5 - in Jamie's help file.]
'
'NOTE:      Currently only used in stretch/shrink forms
'---------------------------------------------------------------------------------------
Public Sub getOrigWindow(frm As Access.Form)

On Error Resume Next

    OrigWindow.Height = frm.WindowHeight
    OrigWindow.Width = frm.WindowWidth
    
    lngOrigWidth = OrigWindow.Width
    lngOrigHeight = OrigWindow.Height

End Sub

Public Sub GetResizedWindow(frm As Access.Form)
'---------------------------------------------------------------------------------------
' Procedure :       GetResizedWindow
' DateTime  :       28/11/2022
' Author    :         Colin Riddington
' Purpose   :       Routine stores the resized window dimensions
'---------------------------------------------------------------------------------------

On Error Resume Next

    ResizedWindow.Height = frm.WindowHeight
    ResizedWindow.Width = frm.WindowWidth
    
    lngNewWidth = ResizedWindow.Width
    lngNewHeight = ResizedWindow.Height
    
    'Debug.Print "RW:" & ResizedWindow.Width, ResizedWindow.Height

End Sub

Public Function GetStretchFactor() As Single

'---------------------------------------------------------------------------------------
' Procedure :       GetStretchFactor
' DateTime  :       28/11/2022
' Author    :       Colin Riddington
' Purpose   :       Routine determines the stretch/shrink factor in forms using this feature
'                          Not used elsewhere
'---------------------------------------------------------------------------------------

'Original idea was to get the smaller change in case form not resized evenly in each direction
'However this seems to work well enough - v3.90

 If lngNewWidth / lngOldWidth > 1 Then 'stretch
       GetStretchFactor = lngNewWidth / lngOldWidth 'use width
 Else 'shrink
       GetStretchFactor = lngNewHeight / lngOldHeight
 End If

End Function

'---------------------------------------------------------------------------------------
' Procedure :       RestoreWindow
' DateTime  :       27/01/2003
' Author    :          Jamie Czernik
' Purpose   :         Routine restores the original window dimensions call it when form closes.
'                           Intended for use in conjunction with GetOrigWindow
'                           Call it in Form_Close()
'                           [More info in "Important Points" - point 5 - in help file.]

'                           Not currently used in my apps but retained for possible use by others
'---------------------------------------------------------------------------------------
Public Sub RestoreWindow()

On Error Resume Next

    Access.DoCmd.MoveSize , , OrigWindow.Width, OrigWindow.Height
    Access.DoCmd.Save
    
End Sub

'---------------------------------------------------------------------------------------
' Procedure :  ZoomForm
' DateTime  :  10/03/2019
' Author    :     Colin Riddington
' Purpose   :   Zoom form in/out by a specified multiplier sngZoom
'                     Routine should be called after the form has been resized
'                     e.g. using a combo or slider control
'---------------------------------------------------------------------------------------
Public Sub ZoomForm(frm As Access.Form, sngZoom As Single)

Dim rectWindow As tRect
Dim lngWidth As Long
Dim lngHeight As Long
Dim sngFactor As Single

On Error Resume Next
    GetScreenResolution
    sngFactor = GetFactor 'local function
    If NewRes.Width <> DESIGN_HORZRES Then 'no resize necessary
        Resize sngFactor * sngZoom, frm  'local procedure
        
        'the following code controls the positioning of pop-up forms
        'but only if the form tag is null.  This allows more control where it causes a problem
        If IsZoomed(frm.hWnd) = 0 Then            'Don't change window settings for maximised form.
            Access.DoCmd.RunCommand acCmdAppMaximize    'Max Access Window
            Call GetWindowRect(frm.hWnd, rectWindow)
            With rectWindow
                lngWidth = .Right - .Left
                lngHeight = .Bottom - .Top
            End With
            If Nz(frm.Tag, 1) <> 1 Then
                Call MoveWindow(frm.hWnd, ((NewRes.Width - _
                (sngFactor * lngWidth)) / 2) - getLeftOffset, _
                ((NewRes.Height - (sngFactor * lngHeight)) / 2) - getTopOffset, _
                lngWidth * sngFactor * sngZoom, lngHeight * sngFactor * sngZoom, 1)
            End If
        End If
    End If
    
    'Modification by Colin Riddington 13/03/2019
    'UseMDIMode property =1 (overlapping windows) or = 0 (tabbed documents)
    'next section fixes display issue for users of tabbed documents (MDIMode=0)
    If CurrentDb.Properties("UseMDIMode") = 0 Then
        MaximizeNavigationPane
        DoEvents
        MinimizeNavigationPane
    End If
    
    
End Sub

'---------------------------------------------------------------------------------------
' Procedure :               ReportResize
' DateTime  :              18/02/2022
' Version :                  3.73
' Author    :                Colin Riddington
' Purpose   :               Allows resizing of report opened as a subform
'---------------------------------------------------------------------------------------

Public Sub ReportResize(rpt As Access.Report)

Dim ctl As Access.Control

On Error Resume Next
    
    'Resize height for each section:
    With rpt
        .Width = .Width * GetFactor
       .Section(Access.acHeader).Height = .Section(Access.acHeader).Height * GetFactor
       .Section(Access.acDetail).Height = .Section(Access.acDetail).Height * GetFactor
       .Section(Access.acFooter).Height = .Section(Access.acFooter).Height * GetFactor
    End With
        
    'Resize and locate each control:
    For Each ctl In rpt.Controls
        If ctl.ControlType <> acPage Then 'Ignore pages in TAB controls
            With ctl
                .Left = .Left * GetFactor
                .Top = .Top * GetFactor
                
                .Height = .Height * GetFactor
                .Width = .Width * GetFactor
                
               'Several of the control types below are unlikley to be used in reports
                'Fix certain combo box, list box and tab control properties:
                Select Case .ControlType
                    Case acLabel, acCommandButton, acTextBox, acToggleButton '
                         'increase font size proportionately
                         .FontSize = .FontSize * GetFactor
                    Case acListBox
                        'as above & increase width of each column proportionately
                         .FontSize = .FontSize * GetFactor
                        .ColumnWidths = adjustColumnWidths(.ColumnWidths, GetFactor)
                    Case acComboBox
                        'as above & increase list width proportionately
                        .FontSize = .FontSize * GetFactor
                        .ColumnWidths = adjustColumnWidths(.ColumnWidths, GetFactor)
                        .ListWidth = .ListWidth * GetFactor
                    Case acTabCtl
                        'as for textbox & increase width/height of each tab proportionately
                        .FontSize = .FontSize * GetFactor
                        .TabFixedWidth = .TabFixedWidth * GetFactor
                        .TabFixedHeight = .TabFixedHeight * GetFactor
                    Case Else
                        'no code needed for other control types
                        'acRectangle, acCheckBox, acImage, acLine, acPageBreak, acSubform
                        'acOptionButton, acOptionGroup, acObjectFrame, acBoundObjectFrame
                        
                End Select
                
            End With
        End If
    Next ctl
    
End Sub

Function IsSubform(ByVal frm As Access.Form) As Boolean

'---------------------------------------------------------------------------------------
' Procedure :            IsSubform
' DateTime  :            11/09/2014
' Author :                  Colin Riddington
' Purpose :                Checks if form opened as a subform
'---------------------------------------------------------------------------------------
   Dim bHasParent As Boolean

   On Error GoTo NotASubform

   ' If opened not as a subform, accessing
   ' the Parent property raises an error:
   bHasParent = Not (frm.Parent Is Nothing)

   IsSubform = True
   Exit Function

NotASubform:
   IsSubform = False
End Function

Sub GetOriginalSize(frm As Access.Form)

'---------------------------------------------------------------------------------------
' Procedure :            GetOriginalSize
' DateTime  :            23/12/2022
' Author :                  Colin Riddington
' Purpose :                Used to get original size of form & its sections before reszing using a mouse
'---------------------------------------------------------------------------------------

On Error Resume Next

    DoCmd.Restore

    'get form dimensions on load
    getOrigWindow frm

    'get height of each section
    lngHeaderHeight = frm.FormHeader.Height
    lngFooterHeight = frm.FormFooter.Height
    lngDetailHeight = frm.Detail.Height

    'on form load set both lngNew & lngOld values to equal lngOrig values
    lngNewWidth = lngOrigWidth
    lngOldWidth = lngOrigWidth

    lngNewHeight = lngOrigHeight
    lngOldHeight = lngOrigHeight

End Sub

Sub StretchShrink(frm As Access.Form)

'---------------------------------------------------------------------------------------
' Procedure :            StretchShrink
' DateTime  :            23/12/2022
' Author :                  Colin Riddington
' Purpose :                Used to maintain width:height ratio when the form is resized using a mouse
'---------------------------------------------------------------------------------------

On Error Resume Next
    
    Dim sngStretchWidth As String, sngStretchHeight As String
    Dim blnRestore As Boolean

    Application.Echo False

    'ensures width & height are changed in proportion as form is stretched / shrunk
    GetResizedWindow frm

    sngStretchWidth = lngNewWidth / lngOldWidth
    sngStretchHeight = lngNewHeight / lngOldHeight

    If sngStretchWidth >= sngStretchHeight Then
        If sngStretchWidth >= 1 Then
            sngStretchHeight = sngStretchWidth
            lngNewHeight = lngOldHeight * sngStretchHeight
        Else
            sngStretchWidth = sngStretchHeight
            lngNewWidth = lngOldWidth * sngStretchWidth
        End If

    Else
        If sngStretchHeight >= 1 Then
            sngStretchWidth = sngStretchHeight
            lngNewWidth = lngOldWidth * sngStretchWidth
        Else
            sngStretchHeight = sngStretchWidth
            lngNewHeight = lngOldHeight * sngStretchHeight
        End If

    End If

    DoEvents
    'move to top left & update form size
    DoCmd.MoveSize 0, 0, lngNewWidth, lngNewHeight

    'adjust height of each section in proportion
    If GetStretchFactor <> 1 Then 'resize necessary
       frm.Detail.Height = lngDetailHeight * GetStretchFactor
       frm.FormHeader.Height = lngHeaderHeight * GetStretchFactor
       frm.FormFooter.Height = lngFooterHeight * GetStretchFactor

       'resize all controls
       Resize GetStretchFactor, frm

     '  Resize GetStretchFactor, subfrmControlTypes.Form

    End If

    'set old values to equal new values (used in GetStretchFactor)
    lngOldWidth = lngNewWidth
    lngOldHeight = lngNewHeight

    Application.Echo True

End Sub

Public Function IsFormDialog(frm As Access.Form) As Boolean

'---------------------------------------------------------------------------------------
' Procedure :            IsFormDialog
' DateTime  :            23/03/2017
' Author :                  Philipp Stiefel
' Purpose :                Checks if form is opened as Dialog
'---------------------------------------------------------------------------------------
    Dim hWnd As Long
    Dim lngStyle As Long
    
    hWnd = frm.hWnd
    lngStyle = GetWindowLong(hWnd, GWL_EXSTYLE)
    IsFormDialog = CBool((lngStyle And WS_EX_DLGMODALFRAME) = WS_EX_DLGMODALFRAME)
End Function

Public Function MinimizeNavigationPane()

On Error GoTo ErrHandler

    DoCmd.NavigateTo "acNavigationCategoryObjectType"
    DoCmd.Minimize
        
Exit_ErrHandler:
    Exit Function
    
ErrHandler:
    MsgBox "Error " & Err.Number & " in MinimizeNavigationPane routine : " & Err.Description, vbOKOnly + vbCritical
    Resume Exit_ErrHandler

End Function

Public Function MaximizeNavigationPane()

On Error GoTo ErrHandler

    DoCmd.NavigateTo "acNavigationCategoryObjectType"
    DoCmd.Maximize
        
Exit_ErrHandler:
    Exit Function
    
ErrHandler:
    MsgBox "Error " & Err.Number & " in MaximizeNavigationPane routine : " & Err.Description, vbOKOnly + vbCritical
    Resume Exit_ErrHandler

End Function


